@@ -1,6 +1,8 @@
 from abc import ABC, abstractmethod
 from dataclasses import KW_ONLY, dataclass
-from typing import Callable, Dict
+from typing import Callable, Dict, Union
+
+from torch import NoneType
 
 from ..types.type_signature import (
     LambdaTypeSignature,
@@ -13,31 +15,45 @@ class Production(ABC):
     """
     Represents a production rule in a simple s-expression grammar.
 
-    Has a symbol, a type signature, and a function that represents the fn represented.
+    Has a symbol, a type signature, and a function that computes the
+    semantics of the production.
     """
 
     @abstractmethod
-    def base_symbol(self):
-        pass
+    def base_symbol(self) -> str:
+        """
+        Get the "base" symbol of this production, without the index.
+        """
 
     @abstractmethod
-    def get_index(self):
-        pass
-
-    def get_numerical_index(self):
+    def get_index(self) -> Union[int, NoneType]:
         """
-        Like get_index, but returns 0 if the index is None.
+        Get the index of this production; this is used to disambiguate
+        productions with the same base symbol. These are created by
+        the :py:class:`neurosym.dsl.DSLFactory` either by
+        :py:meth:`neurosym.dsl.DSLFactory.lambdas` or by
+        type variables that appear on only the left or right side of a type signature
+        being enumerated as templates.
+        """
+
+    def get_numerical_index(self) -> int:
+        """
+        Like :py:meth:`get_index`, but returns 0 if the index is None.
         """
         idx = self.get_index()
         return 0 if idx is None else idx
 
     @abstractmethod
-    def with_index(self, index):
-        pass
-
-    def symbol(self):
+    def with_index(self, index: int) -> "Production":
         """
-        Return the symbol of this production.
+        A copy of this production with the given index.
+        """
+
+    def symbol(self) -> str:
+        """
+        Return the symbol of this production. This is
+        the base symbol if the index is None, otherwise
+        it is the base symbol followed by an underscore and the index.
         """
         if self.get_index() is None:
             return self.base_symbol()
@@ -57,9 +73,14 @@ class Production(ABC):
         """
 
     @abstractmethod
-    def apply(self, dsl, state, children):
+    def apply(self, dsl, state, children, environment):
         """
         Apply this production to the given children.
+
+        :param dsl: the DSL to apply relative to
+        :param state: the state dictionary for this production
+        :param children: the children of this production, as InitializedSExpression objects
+        :param environment: the current de-bruijn environment. environment[i] corresponds to variable i.
         """
 
     @abstractmethod
@@ -79,24 +100,35 @@ class Production(ABC):
 
 
 class FunctionLikeProduction(Production):
+    """
+    Production that acts like a function, i.e., evaluates each of its children
+    as values and then applies a function to them.
+    """
+
     @abstractmethod
-    def evaluate(self, dsl, state, inputs):
+    def evaluate(self, dsl, state, inputs, environment):
         """
         Return the resulting pytorch expression of computing this function on the inputs
-            Takes in the state of the production, which is the result of initialize().
+        Takes in the state of the production, which is the result of
+        :py:meth:`neurosym.DSL.initialize`
 
         Effectively a form of denotation semantics.
         """
 
-    def apply(self, dsl, state, children):
-        """
-        Apply this production to the given children.
-        """
-        return self.evaluate(dsl, state, [dsl.compute(x) for x in children])
+    def apply(self, dsl, state, children, environment):
+        return self.evaluate(
+            dsl, state, [dsl.compute(x, environment) for x in children], environment
+        )
 
 
 @dataclass
 class ConcreteProduction(FunctionLikeProduction):
+    """
+    Represents a production rule in a simple s-expression grammar, that does
+    not have any parameters. This is added to the DSL by the :py:class:`neurosym.DSLFactory`
+    when :py:meth:`neurosym.DSLFactory.production` is called.
+    """
+
     _symbol: str
     _type_signature: TypeSignature
     _compute: Callable[..., object]
@@ -121,8 +153,8 @@ class ConcreteProduction(FunctionLikeProduction):
         del dsl
         return {}
 
-    def evaluate(self, dsl, state, inputs):
-        del dsl
+    def evaluate(self, dsl, state, inputs, environment):
+        del dsl, environment
         assert state == {}
         try:
             return self._compute(*inputs)
@@ -137,6 +169,12 @@ class ConcreteProduction(FunctionLikeProduction):
 
 @dataclass
 class LambdaProduction(Production):
+    """
+    This production represents a lambda function. This is added automatically
+    to the DSL by the :py:class:`neurosym.DSLFactory` when :py:meth:`neurosym.DSLFactory.lambdas`
+    is called.
+    """
+
     _unique_id: int
     _type_signature: LambdaTypeSignature
 
@@ -160,15 +198,16 @@ class LambdaProduction(Production):
         del dsl
         return {}
 
-    def apply(self, dsl, state, children):
+    def apply(self, dsl, state, children, environment):
         """
-        Apply this production to the given children.
+        Apply the lambda production to its child, creating
+        a function with the child as the body.
         """
         # pylint: disable=cyclic-import
         from neurosym.dsl.lambdas import LambdaFunction
 
         [body] = children
-        return LambdaFunction.of(dsl, body, self._type_signature)
+        return LambdaFunction.of(dsl, body, self._type_signature, environment)
 
     def render(self):
         return f"{self.symbol():>15} :: {self._type_signature.render()}"
@@ -176,6 +215,12 @@ class LambdaProduction(Production):
 
 @dataclass
 class VariableProduction(Production):
+    """
+    A production that represents a de Bruijn variable. This is added automatically
+    to the DSL by the :py:class:`neurosym.DSLFactory` when
+    :py:meth:`neurosym.DSLFactory.lambdas` is called.
+    """
+
     _unique_id: int
     _type_signature: VariableTypeSignature
 
@@ -195,11 +240,9 @@ class VariableProduction(Production):
         del dsl
         return {}
 
-    def apply(self, dsl, state, children):
-        """
-        Apply this production to the given children.
-        """
-        raise NotImplementedError
+    def apply(self, dsl, state, children, environment):
+        del dsl, state, children
+        return environment[self._type_signature.index_in_env].object_value
 
     def render(self):
         return f"{self.symbol():>15} :: {self._type_signature.render()}"
@@ -211,7 +254,30 @@ class VariableProduction(Production):
 
 @dataclass
 class ParameterizedProduction(ConcreteProduction):
+    """
+    Like a concrete production, but with some parameters that need to be initialized.
+
+    This is added to the DSL by the :py:class:`neurosym.DSLFactory` when
+    :py:meth:`neurosym.DSLFactory.production` is called with the `initializers` argument.
+    """
+
     initializers: Dict[str, Callable[[], object]]
+    provide_enviroment: Union[NoneType, str] = None
+
+    @classmethod
+    def of(
+        cls, symbol, type_signature, compute, initializers=None, provide_enviroment=None
+    ):
+        if not initializers and provide_enviroment is None:
+            return ConcreteProduction(symbol, type_signature, compute)
+        # pylint: disable=unexpected-keyword-arg
+        return cls(
+            symbol,
+            type_signature,
+            compute,
+            initializers=initializers,
+            provide_enviroment=provide_enviroment,
+        )
 
     def with_index(self, index):
         # pylint: disable=unexpected-keyword-arg
@@ -227,9 +293,12 @@ class ParameterizedProduction(ConcreteProduction):
         del dsl
         return {k: v() for k, v in self.initializers.items()}
 
-    def evaluate(self, dsl, state, inputs):
+    def evaluate(self, dsl, state, inputs, environment):
         del dsl
-        return self._compute(*inputs, **state)
+        kwargs = state.copy()
+        if self.provide_enviroment is not None:
+            kwargs[self.provide_enviroment] = environment
+        return self._compute(*inputs, **kwargs)
 
     def render(self):
         lhs = f"{self.symbol()}[{', '.join(self.initializers)}]"
