@@ -1,3 +1,3 @@
-from . import dreamcoder, near
+from . import dreamcoder, near, simple_dreamcoder
 from .basic_arith import basic_arith_dsl
 from .mutable_arith_combinators import mutable_arith_combinators_dsl
