@@ -1,6 +1,3 @@
-"""
-RNN example
-"""
 import torch
 from torch import nn
 
@@ -10,37 +7,46 @@ from ..operations.basic import ite_torch
 from ..operations.lists import fold_torch, map_torch
 
 
-def example_rnn_dsl(L, O):
-    dslf = DSLFactory(L=L, O=O, max_overall_depth=5)
+def example_rnn_dsl(input_size, output_size):
+    """
+    An example DSL for RNN-like sequence-to-sequence tasks.
+
+    :param input_size: Size of the input vectors
+    :param output_size: Size of the output vectors
+    """
+    dslf = DSLFactory(L=input_size, O=output_size, max_overall_depth=5)
     dslf.typedef("fL", "{f, $L}")
 
-    dslf.concrete("add", "() -> ($fL, $fL) -> $fL", lambda: lambda x, y: x + y)
-    dslf.concrete("mul", "() -> ($fL, $fL) -> $fL", lambda: lambda x, y: x * y)
-    dslf.concrete(
+    dslf.production("add", "() -> ($fL, $fL) -> $fL", lambda: lambda x, y: x + y)
+    dslf.production("mul", "() -> ($fL, $fL) -> $fL", lambda: lambda x, y: x * y)
+    dslf.production(
         "fold", "((#a, #a) -> #a) -> [#a] -> #a", lambda f: lambda x: fold_torch(f, x)
     )
-    dslf.concrete(
+    dslf.production(
         "sum", "() -> $fL -> f", lambda: lambda x: torch.sum(x, dim=-1).unsqueeze(-1)
     )
-    dslf.parameterized(
-        "linear", "() -> $fL -> $fL", lambda lin: lin, dict(lin=lambda: nn.Linear(L, L))
+    dslf.production(
+        "linear",
+        "() -> $fL -> $fL",
+        lambda lin: lin,
+        dict(lin=lambda: nn.Linear(input_size, input_size)),
     )
-    dslf.parameterized(
+    dslf.production(
         "output",
         "(([$fL]) -> [$fL]) -> [$fL] -> [{f, $O}]",
         lambda f, lin: lambda x: lin(f(x)),
-        dict(lin=lambda: nn.Linear(L, O)),
+        dict(lin=lambda: nn.Linear(input_size, output_size)),
     )
-    dslf.concrete(
+    dslf.production(
         "ite",
         "(#a -> f, #a -> #a, #a -> #a) -> #a -> #a",
         ite_torch,
     )
-    dslf.concrete(
+    dslf.production(
         "map", "(#a -> #b) -> [#a] -> [#b]", lambda f: lambda x: map_torch(f, x)
     )
 
-    dslf.concrete(
+    dslf.production(
         "compose", "(#a -> #b, #b -> #c) -> #a -> #c", lambda f, g: lambda x: g(f(x))
     )
 
