@@ -18,22 +18,62 @@ from neurosym.program_dist.tree_distribution.preorder_mask.type_preorder_mask im
 from neurosym.program_dist.tree_distribution.tree_distribution import TreeDistribution
 from neurosym.programs.s_expression import SExpression
 from neurosym.types.type import Type
+from neurosym.utils.documentation import internal_only
 
 from .tree_distribution.tree_distribution import TreeProgramDistributionFamily
 
 
 @dataclass
 class BigramProgramDistribution:
+    """
+    Represents a bigram program distribution. This is a distribution over
+    programs that is conditioned on the parent and the position of the child
+    in the parent.
+
+    :field dist_fam: The family of distributions that this distribution is from.
+    :field distribution: A numpy array of shape ``(num_symbols, max_arity, num_symbols)``
+        where ``num_symbols`` is the number of symbols in the DSL and ``max_arity`` is
+        the maximum arity of any symbol. The value at ``(i, j, k)`` is the probability
+        of symbol ``k`` appearing in position ``j`` of symbol ``i``.
+    """
+
     dist_fam: "BigramProgramDistributionFamily"
     distribution: np.ndarray
+    _disable_validation: bool = False
 
     def __post_init__(self):
         assert self.distribution.ndim == 3
         assert self.distribution.shape[0] == self.distribution.shape[2]
+        if not self._disable_validation:
+            # check that everything reachable has a distribution
+            distro_sums = self.distribution.sum(-1)
+            sym_arities = self.dist_fam.sym_arities
+            seen, fringe = set(), [sym_arities.index(("<root>", 1))]
+            while fringe:
+                idx = fringe.pop()
+                if idx in seen:
+                    continue
+                seen.add(idx)
+                for child_idx in range(sym_arities[idx][1]):
+                    assert np.allclose(distro_sums[idx, child_idx], 1.0), (
+                        f"The reachable transition ({sym_arities[idx][0]}, {child_idx})'s"
+                        f" distribution has a sum of {distro_sums[idx, child_idx]} instead of 1.0"
+                    )
 
     def bound_minimum_likelihood(
         self, min_likelihood: float, symbol_mask: np.ndarray = None
     ):
+        """
+        Ensure that the probability of any symbol is at least ``min_likelihood``,
+        approximately, except symbols that have been masked out. This is done
+        by setting the probability of any symbol that is not masked out to be at
+        least ``min_likelihood``, then renormalizing the distribution.
+
+        :param min_likelihood: The minimum likelihood that any symbol should have.
+        :param symbol_mask: A boolean array of shape ``(num_symbols,)`` that is
+            False for symbols that should not be affected by the minimum likelihood
+            bound. If None, all symbols are affected.
+        """
         assert 0 <= min_likelihood <= 1
         if symbol_mask is not None:
             assert (
@@ -61,6 +101,12 @@ class BigramProgramDistribution:
         return mask_square
 
     def mix_with_other(self, other: "BigramProgramDistribution", weight_other: float):
+        """
+        Mix this distribution with another distribution. This is done by taking
+        a weighted average of the two distributions, where the weight of the other
+        distribution is ``weight_other`` and the weight of this distribution is
+        ``1 - weight_other``.
+        """
         # pylint: disable=self-cls-assignment
         assert 0 <= weight_other <= 1
         symbols_this = self.dist_fam.symbols()
@@ -93,7 +139,7 @@ class BigramProgramDistribution:
 
 
 @dataclass
-class BigramProgramDistributionBatch:
+class _BigramProgramDistributionBatch:
     dist_fam: "BigramProgramDistributionFamily"
     distribution_batch: np.ndarray
 
@@ -113,18 +159,49 @@ class BigramProgramDistributionBatch:
 
 @dataclass
 class BigramProgramCounts:
+    """
+    Represents counts of bigram programs, both the "numerator" counts
+        (the number of times a symbol appears in a given context), and
+        the "denominator" counts (the number of times a symbol could
+        have appeared in that context).
+
+    :field numerators: A map from context ``(parent_sym, parent_child_idx)`` to a map from
+        ``child_sym`` to ``count``.
+    :field denominators: A map from context ``(parent_sym, parent_child_idx)`` to a map from
+        ``child_syms`` to ``count``. The ``child_syms`` are the set of symbols that could have
+        appeared in that context.
+    """
+
     # map from (parent_sym, parent_child_idx) to map from child_sym to count
     numerators: Dict[Tuple[Tuple[int, int], ...], Dict[int, int]]
     # map from (parent_sym, parent_child_idx) to map from potential child_sym values to count
     denominators: Dict[Tuple[Tuple[int, int], ...], Dict[Tuple[int, ...], int]]
 
     def add_to_numerator_array(self, arr, batch_idx):
+        """
+        Add the numerator counts to the given array at the given batch index.
+
+        :param arr: An array of counts, to be mutated. Must be indexable as
+            ``arr[batch_idx, parent_sym, parent_child_idx, child_sym]``.
+            A reference to this array is returned.
+        :param batch_idx: The batch index to add the counts to.
+        """
         for [(parent_sym, parent_child_idx)], children in self.numerators.items():
             for child_sym, count in children.items():
                 arr[batch_idx, parent_sym, parent_child_idx, child_sym] = count
         return arr
 
     def add_to_denominator_array(self, arr, batch_idx, backmap):
+        """
+        Add the denominator counts to the given array at the given batch index.
+
+        :param arr: An array of counts, to be mutated. Must be indexable as
+            ``arr[batch_idx, parent_sym, parent_child_idx, denominator_id]``.
+            A reference to this array is returned.
+        :param batch_idx: The batch index to add the counts to.
+        :param backmap: A mapping from the set of child symbols to the index in the
+            denominator array.
+        """
         for [(parent_sym, parent_child_idx)], children in self.denominators.items():
             for child_syms, count in children.items():
                 key = batch_idx, parent_sym, parent_child_idx, backmap[child_syms]
@@ -134,10 +211,22 @@ class BigramProgramCounts:
 
 @dataclass
 class BigramProgramCountsBatch:
+    """
+    Like BigramProgramCounts, but batched, that is, it contains a list of
+    BigramProgramCounts objects.
+
+    :field dist_fam: The family of distributions that these counts are for.
+    :field counts: A list of BigramProgramCounts objects.
+    """
+
     dist_fam: "BigramProgramDistributionFamily"
     counts: List[BigramProgramCounts]
 
     def numerators(self, num_symbols, max_arity):
+        """
+        See ``BigramProgramCounts.numerators`` for details. This is a batched version
+        that creates and returns a numpy array of counts.
+        """
         numerators = np.zeros(
             (len(self.counts), num_symbols, max_arity, num_symbols), dtype=np.int32
         )
@@ -146,6 +235,11 @@ class BigramProgramCountsBatch:
         return numerators
 
     def denominators(self, num_symbols, max_arity):
+        """
+        See ``BigramProgramCounts.denominators`` for details. This is a batched version
+        that creates and returns a numpy array of counts, along with the denominator index
+        sets that correspond to the last axis of the array.
+        """
         denominator_keys = {
             key
             for counts in self.counts
@@ -164,18 +258,41 @@ class BigramProgramCountsBatch:
         return denominators, denominator_keys
 
     def to_distribution(self, num_symbols, max_arity):
+        """
+        Convert these counts to a ``BigramProgramDistribution`` object. This is a
+        batched version that creates and returns a ``BigramProgramDistributionBatch`` object.
+
+        Note that we are not handling denominators here, as this is just a simple conversion
+        from counts to probabilities. This is probably not fully correct, as we should be
+        using a more sophisticated algorithm to fit the parameters to the counts. However,
+        this is a relatively simple way to get a distribution from counts.
+        """
+        # TODO(kavigupta): Implement a proper fitting algorithm here.
         numerators = self.numerators(num_symbols, max_arity)
 
-        # We do not need to handle denominators here, as this is just
-        # a simple conversion from counts to probabilities, and we do
-        # not need to handle the case where the denominator is 0.
-
-        return BigramProgramDistributionBatch(
-            self.dist_fam, counts_to_probabilities(numerators)
+        return _BigramProgramDistributionBatch(
+            self.dist_fam, _counts_to_probabilities(numerators)
         )
 
 
 class BigramProgramDistributionFamily(TreeProgramDistributionFamily):
+    """
+    A family of bigram program distributions. These are TreeProgramDistributions
+    that are conditioned on just the parent and the position of the child
+    in the parent. This is a kind of TreeProgramDistributionFamily.
+
+    :param dsl: The DSL that this family is for.
+    :param valid_root_types: The types that are valid as roots of programs.
+    :param additional_preorder_masks: A tuple of functions that take a TreeDistribution
+        and return a PreorderMask. These masks are used to filter the set of possible
+        symbols that can appear in a given context.
+    :param include_type_preorder_mask: Whether to include a type preorder mask in the
+        set of masks that are used to filter the set of possible symbols that can appear
+        in a given context.
+    :param node_ordering: The node ordering to use when traversing the tree. This
+        determines the order in which the children of a node are considered.
+    """
+
     def __init__(
         self,
         dsl: DSL,
@@ -190,49 +307,52 @@ class BigramProgramDistributionFamily(TreeProgramDistributionFamily):
         if valid_root_types is not None:
             dsl = dsl.with_valid_root_types(valid_root_types)
         self._dsl = dsl
-        self._symbols, self._arities, self._valid_mask = bigram_mask(dsl)
+        self._symbols, self._arities, self._valid_mask = _bigram_mask(dsl)
         self._max_arity = max(self._arities)
         self._symbol_to_idx = {sym: i for i, sym in enumerate(self._symbols)}
         self._additional_preorder_masks = additional_preorder_masks
         self._include_type_preorder_mask = include_type_preorder_mask
         self._node_ordering = node_ordering
 
+    @property
+    def sym_arities(self) -> List[int]:
+        return list(zip(self._symbols, self._arities))
+
     def underlying_dsl(self) -> DSL:
         return self._dsl
 
     def parameters_shape(self) -> List[int]:
         return self._valid_mask.shape
 
-    def normalize_parameters(
-        self, parameters: torch.Tensor, *, logits: bool, neg_inf=-float("inf")
-    ) -> torch.Tensor:
+    def _normalize_parameters(self, parameters: torch.Tensor) -> torch.Tensor:
+        """
+        Apply a mask to the parameters, then normalize them to be a valid
+        probability distribution. If ``logits`` is True, the parameters are
+        returned as logits, otherwise they are returned as probabilities.
+        """
         parameters = parameters.clone()
         mask = torch.tensor(self._valid_mask, device=parameters.device)[None].repeat(
             parameters.shape[0], 1, 1, 1
         )
         parameters[~mask] = -float("inf")
-        if logits:
-            parameters = parameters.log_softmax(-1)
-            parameters[~mask] = neg_inf
-        else:
-            parameters = parameters.softmax(-1)
-            parameters[~mask] = 0
+        parameters = parameters.softmax(-1)
+        parameters[~mask] = 0
         return parameters
 
     def with_parameters(
         self, parameters: torch.Tensor
-    ) -> BigramProgramDistributionBatch:
+    ) -> _BigramProgramDistributionBatch:
         assert (
             parameters.shape[1:] == self.parameters_shape()
         ), f"Expected {self.parameters_shape()}, got {parameters.shape[1:]}"
-        parameters = self.normalize_parameters(parameters, logits=False)
-        return BigramProgramDistributionBatch(self, parameters.detach().cpu().numpy())
+        parameters = self._normalize_parameters(parameters)
+        return _BigramProgramDistributionBatch(self, parameters.detach().cpu().numpy())
 
     def count_programs(self, data: List[List[SExpression]]) -> BigramProgramCountsBatch:
         tree_dist = self.tree_distribution_skeleton
         all_counts = []
         for programs in data:
-            numerators, denominators = count_programs(tree_dist, programs)
+            numerators, denominators = _count_programs(tree_dist, programs)
             all_counts.append(
                 BigramProgramCounts(numerators=numerators, denominators=denominators)
             )
@@ -246,70 +366,67 @@ class BigramProgramDistributionFamily(TreeProgramDistributionFamily):
     def parameter_difference_loss(
         self, parameters: torch.tensor, actual: BigramProgramCountsBatch
     ) -> torch.float32:
-        """
-        Let
-            p be a program
-            g be a bigram context (i.e. parent +, in position 2)
-            s be a symbol
-            d be the "denominator" (which is the set of *possible* symbols s'
-                that could have appeared in this context g).
-        The ngram `g` is really the context that `s` appears in. So for example
-            `(+ (- 1 2) 3)` if we're thinking about the symbol s=`-` its bigram g=`(+, 0)`
-            indicating it's the first argument of a "+". This is the bigram notion from
-            DreamCoder where it's not just the parent (analogous to "previous token" in NLP)
-            but also includes which child of the parent we are. In general, we also filter
-            on other characteristics (e.g., type checking), not just ngrams. See the
-            preorder mask for more details.
+        # Let
+        #     p be a program
+        #     g be a bigram context (i.e. parent +, in position 2)
+        #     s be a symbol
+        #     d be the "denominator" (which is the set of *possible* symbols s'
+        #         that could have appeared in this context g).
+        # The ngram ``g`` is really the context that ``s`` appears in. So for example
+        #     ``(+ (- 1 2) 3)`` if we're thinking about the symbol s=``-`` its bigram g=``(+, 0)``
+        #     indicating it's the first argument of a "+". This is the bigram notion from
+        #     DreamCoder where it's not just the parent (analogous to "previous token" in NLP)
+        #     but also includes which child of the parent we are. In general, we also filter
+        #     on other characteristics (e.g., type checking), not just ngrams. See the
+        #     preorder mask for more details.
 
-        In this particular example, `d` would be the set of symbols that *could* have
-            appeared in that same position `g`, for example `1` or `+` or `-` or anything
-            else that type checks. This is called the denominator because the
-            probability of choosing `s` among `(s' in d)` is going to be
-                P(s)/sum_{s' in d} P(s')
-            These probabilities will depend on the parameters `theta` of the bigram model
-            (and of course, specifically the unigram it assigns to the context `g`).
+        # In this particular example, ``d`` would be the set of symbols that *could* have
+        #     appeared in that same position ``g``, for example ``1`` or ``+`` or ``-`` or anything
+        #     else that type checks. This is called the denominator because the
+        #     probability of choosing ``s`` among ``(s' in d)`` is going to be
+        #         P(s)/sum_{s' in d} P(s')
+        #     These probabilities will depend on the parameters ``theta`` of the bigram model
+        #     (and of course, specifically the unigram it assigns to the context ``g``).
 
-        (See also math below) Our goal is to compute the loglikelihood of the actual
-            program P under the bigram parameters theta. Which, for a bigram is the sum
-            of the logprob over all subtrees of the symbol `s` context `g` and denominator `d` for
-            that subtree. We can factor this overall sum into two parts: a numerator based on
-            sthe actual symbol `s` and a denominator based on the alternative symbols in `d`
-            for that context `g`.
+        # (See also math below) Our goal is to compute the loglikelihood of the actual
+        #     program P under the bigram parameters theta. Which, for a bigram is the sum
+        #     of the logprob over all subtrees of the symbol ``s`` context ``g`` and denominator ``d`` for
+        #     that subtree. We can factor this overall sum into two parts: a numerator based on
+        #     sthe actual symbol ``s`` and a denominator based on the alternative symbols in ``d``
+        #     for that context ``g``.
 
-        sum_p log P(p | theta)
-            = sum_p sum_{(g, s, d) in p} log P(s | g, theta, d)
-            = sum_p sum_{(g, s, d) in p} log (exp(theta_{g, s}) / sum_{s' in d} exp(theta_{g, s'}))
-            = sum_p sum_{(g, s, d) in p} (theta_{g, s} - log sum_{s' in d} exp(theta_{g, s'}))
-            = [sum_p sum_{(g, s, d) in p} theta_{g, s}]
-                - [sum_p sum_{(g, s, d) in p} log sum_{s' in d} exp(theta_{g, s'})]
-            = [numer] - [denom]
+        # sum_p log P(p | theta)
+        #     = sum_p sum_{(g, s, d) in p} log P(s | g, theta, d)
+        #     = sum_p sum_{(g, s, d) in p} log (exp(theta_{g, s}) / sum_{s' in d} exp(theta_{g, s'}))
+        #     = sum_p sum_{(g, s, d) in p} (theta_{g, s} - log sum_{s' in d} exp(theta_{g, s'}))
+        #     = [sum_p sum_{(g, s, d) in p} theta_{g, s}]
+        #         - [sum_p sum_{(g, s, d) in p} log sum_{s' in d} exp(theta_{g, s'})]
+        #     = [numer] - [denom]
 
-        For each (g,s) instance in the corpus the numerator is the same, and for each (g,d)
-            instance the denominator is the same, so we can instead come up with counts for
-            each of these (calling them `numcount` and `dencount`) and rewrite our sum as a
-            sum over the unique (g,s) and (g,d) instances:
+        # For each (g,s) instance in the corpus the numerator is the same, and for each (g,d)
+        #     instance the denominator is the same, so we can instead come up with counts for
+        #     each of these (calling them ``numcount`` and ``dencount``) and rewrite our sum as a
+        #     sum over the unique (g,s) and (g,d) instances:
 
-        numer
-            = sum_p sum_{(g, s, d) in p} theta_{g, s}
-            = sum_g sum_s numcount_{g, s} theta_{g, s}
-            = (numcount * theta).sum()
+        # numer
+        #     = sum_p sum_{(g, s, d) in p} theta_{g, s}
+        #     = sum_g sum_s numcount_{g, s} theta_{g, s}
+        #     = (numcount * theta).sum()
 
-        denom
-            = sum_p sum_{(g, s, d) in p} log sum_{s' in d} exp(theta_{g, s'})
-            = sum_g sum_d dencount_{g, d} log sum_{s' in d} exp(theta_{g, s'})
+        # denom
+        #     = sum_p sum_{(g, s, d) in p} log sum_{s' in d} exp(theta_{g, s'})
+        #     = sum_g sum_d dencount_{g, d} log sum_{s' in d} exp(theta_{g, s'})
 
-        theta_by_denom(g, s', d) = theta_{g, s'} if s' in d, else -inf
+        # theta_by_denom(g, s', d) = theta_{g, s'} if s' in d, else -inf
 
-        denom
-            = sum_g sum_d dencount_{g, d} log sum_{s'} exp(theta_by_denom{g, s', d})
+        # denom
+        #     = sum_g sum_d dencount_{g, d} log sum_{s'} exp(theta_by_denom{g, s', d})
 
-        agg_theta_by_denom(g, d) = logsumexp(theta_by_denom(g, *, d))
+        # agg_theta_by_denom(g, d) = logsumexp(theta_by_denom(g, *, d))
 
-        denom
-            = sum_g sum_d dencount_{g, d} agg_theta_by_denom(g, d)
-            = (dencount * agg_theta_by_denom).sum()
-
-        """
+        # denom
+        #     = sum_g sum_d dencount_{g, d} agg_theta_by_denom(g, d)
+        #     = (dencount * agg_theta_by_denom).sum()
 
         assert parameters.shape[1:] == self.parameters_shape()
         assert len(parameters.shape) == 4
@@ -339,8 +456,14 @@ class BigramProgramDistributionFamily(TreeProgramDistributionFamily):
         return -(numer - denom)
 
     def uniform(self):
+        """
+        A PCFG which is "uniform" in the sense that given a parent and a position
+        in the parent, all symbols are equally likely. This distribution is not
+        necessarily a proper probability distribution, as many paths suggested
+        by this distribution will be invalid. However, it can be enumerated.
+        """
         return BigramProgramDistribution(
-            self, counts_to_probabilities(self._valid_mask)
+            self, _counts_to_probabilities(self._valid_mask)
         )
 
     def compute_tree_distribution(
@@ -366,11 +489,11 @@ class BigramProgramDistributionFamily(TreeProgramDistributionFamily):
             1,
             dist,
             list(zip(self._symbols, self._arities)),
-            self.compute_preorder_mask,
+            self._compute_preorder_mask,
             self._node_ordering,
         )
 
-    def compute_preorder_mask(self, tree_dist):
+    def _compute_preorder_mask(self, tree_dist):
         masks = []
         if self._include_type_preorder_mask:
             masks.append(TypePreorderMask(tree_dist, self._dsl))
@@ -378,14 +501,18 @@ class BigramProgramDistributionFamily(TreeProgramDistributionFamily):
             masks.append(mask(tree_dist, self._dsl))
         return ConjunctionPreorderMask.of(tree_dist, masks)
 
+    @internal_only
     def mask_invalid(self, distribution):
         return distribution * self._valid_mask
 
-    def symbols(self):
+    def symbols(self) -> List[str]:
+        """
+        Get the symbols this distribution is over.
+        """
         return self._symbols
 
 
-def bigram_mask(dsl):
+def _bigram_mask(dsl):
     symbols = dsl.ordered_symbols(include_root=True)
 
     valid_root_types = dsl.valid_root_types
@@ -416,7 +543,7 @@ def bigram_mask(dsl):
     return symbols, np.array(arities), valid_mask
 
 
-def counts_to_probabilities(counts):
+def _counts_to_probabilities(counts):
     return np.divide(
         counts,
         counts.sum(-1)[..., None],
@@ -425,7 +552,7 @@ def counts_to_probabilities(counts):
     )
 
 
-def count_programs(tree_dist: TreeDistribution, programs: List[SExpression]):
+def _count_programs(tree_dist: TreeDistribution, programs: List[SExpression]):
     """
     Count the productions in the programs, indexed by the path to the node.
     """
@@ -434,7 +561,7 @@ def count_programs(tree_dist: TreeDistribution, programs: List[SExpression]):
     for program in programs:
         preorder_mask = tree_dist.mask_constructor(tree_dist)
         preorder_mask.on_entry(0, 0)
-        accumulate_counts(
+        _accumulate_counts(
             tree_dist,
             program,
             numerators,
@@ -447,7 +574,7 @@ def count_programs(tree_dist: TreeDistribution, programs: List[SExpression]):
     return numerators, denominators
 
 
-def accumulate_counts(
+def _accumulate_counts(
     tree_dist: TreeDistribution,
     program: SExpression,
     numerators: Dict[Tuple[Tuple[int, int], ...], Dict[int, int]],
@@ -468,7 +595,7 @@ def accumulate_counts(
     for j, child in zip(order, [program.children[i] for i in order]):
         new_ancestors = ancestors + ((this_idx, j),)
         new_ancestors = new_ancestors[-tree_dist.limit :]
-        accumulate_counts(
+        _accumulate_counts(
             tree_dist,
             child,
             numerators,
