@@ -34,14 +34,14 @@ class ProgramDistributionFamily(ABC):
     def with_parameters(self, parameters: torch.Tensor) -> ProgramDistributionBatch:
         """
         Initializes a distribution from the given parameters. The parameters
-            should have the shape (batch_size, *self.parameters_shape()).
+        should have the shape ``(batch_size, *self.parameters_shape())``.
         """
 
     @abstractmethod
     def count_programs(self, data: List[List[SExpression]]) -> ProgramCountsTensorBatch:
         """
         For each program, count its components' occurrences in the data. This
-            depends on the type of distribution.
+        depends on the type of distribution.
         """
 
     @abstractmethod
@@ -73,9 +73,9 @@ class ProgramDistributionFamily(ABC):
     ) -> torch.float32:
         """
         Returns the loss between the parameters and actual counts, for
-            several count tensors. Keep in mind that the parameters
-            here should have a batch dimension, and
-            parameters.shape[0] == len(actual).
+        several count tensors. Keep in mind that the parameters
+        here should have a batch dimension, and
+        parameters.shape[0] == len(actual).
         """
 
     @abstractmethod
@@ -101,10 +101,22 @@ class ProgramDistributionFamily(ABC):
         """
         Enumerate all programs using iterative deepening. Yields (program, likelihood).
 
-        Args:
-            dist: The distribution to sample from.
-            chunk_size: The amount of likelihood to consider at once. If this is
-                too small, we will spend a lot of time doing the same work over and
-                over again. If this is too large, we will spend a lot of time
-                doing work that we don't need to do.
+        :param dist: The distribution to sample from.
+        :param min_likelihood: The minimum likelihood to consider.
+        :param chunk_size: The amount of likelihood to consider at once. If this is
+            too small, we will spend a lot of time doing the same work over and
+            over again. If this is too large, we will spend a lot of time
+            doing work that we don't need to do.
+        """
+
+    def compute_likelihood(
+        self, dist: ProgramDistribution, program: SExpression
+    ) -> float:
+        """
+        Computes the log-likelihood of a program under this distribution.
+
+        :param dist: The distribution to compute the likelihood under.
+        :param program: The program to compute the likelihood of.
+
+        :return: The log-likelihood of the program.
         """
