@@ -18,11 +18,21 @@ def collect_preorder_symbols(
     symbol_to_index_fn=None,
 ) -> Iterator[Tuple[SExpression, List[str], PreorderMask]]:
     """
-    Collects the alernate symbols that could have been selected in the tree distribution.
+    Collects the alernate symbols that could have been selected in the tree distribution,
+    and yields the S-Expression, the alternate symbols, and the mask at each node.
+
+    :param s_exp: The S-Expression to collect the alternate symbols for.
+    :param tree_dist: The tree distribution.
+    :param replace_node_midstream: A function that takes the current node, the mask, the
+        position of the node, and the alternate symbols that could have been selected, and
+        returns the new node. This is used to replace the node as it is being visited. The
+        returned node is then visited in place of the original node.
+    :param symbol_to_index_fn: A function that takes the mask and the symbol and returns
+        the index of the symbol in the tree distribution.
     """
     mask = tree_dist.mask_constructor(tree_dist)
     mask.on_entry(0, 0)
-    yield from collect_preorder_symbols_dfs(
+    yield from _collect_preorder_symbols_dfs(
         s_exp,
         tree_dist,
         mask,
@@ -32,7 +42,7 @@ def collect_preorder_symbols(
     )
 
 
-def collect_preorder_symbols_dfs(
+def _collect_preorder_symbols_dfs(
     s_exp: SExpression,
     tree_dist: TreeDistribution,
     mask: PreorderMask,
@@ -61,7 +71,7 @@ def collect_preorder_symbols_dfs(
     order = tree_dist.ordering.order(sym_idx, len(s_exp.children))
     for idx, child in zip(order, [s_exp.children[i] for i in order]):
         new_parents = (parents + ((sym_idx, idx),))[-tree_dist.limit :]
-        yield from collect_preorder_symbols_dfs(
+        yield from _collect_preorder_symbols_dfs(
             child,
             tree_dist,
             mask,
@@ -80,6 +90,15 @@ def annotate_with_alternate_symbols(
     """
     Annotates the S-Expression with the alternate symbols that could have been
     selected in the tree distribution.
+
+    Exists mostly for testing purposes.
+
+    :param s_exp: The S-Expression to annotate.
+    :param tree_dist: The tree distribution.
+    :param summary_fn: A function that takes the chosen symbol and the alternate
+        symbols and returns the new symbol.
+
+    :returns: The annotated S-Expression.
     """
     preorder_symbols = [
         (node, alts) for node, alts, _ in collect_preorder_symbols(s_exp, tree_dist)
