@@ -25,7 +25,16 @@ class TreeDistribution:
     Distribution over SExpressions as trees.
 
     Internally, we represent the productions in the language as integers, which we
-        call indices.
+    call indices.
+
+    :param limit: The maximum number of parent nodes that can be conditioned on.
+    :param distribution: A dictionary mapping paths in the tree to lists of (production index, likelihood)
+        pairs. The path is a tuple of tuples of (ancestor index, position), which is the path to the current
+        node, with the most immediate ancestor at the end.
+    :param symbols: A list of (symbol, arity) pairs, where arity is the number of children the symbol has.
+        The root symbol should be at index 0.
+    :param mask_constructor: A function that constructs a preorder mask for the tree distribution.
+    :param node_ordering: A function that constructs a node ordering for the tree distribution.
     """
 
     limit: int
@@ -41,14 +50,41 @@ class TreeDistribution:
     # Node ordering
     node_ordering: Callable[["TreeDistribution"], NodeOrdering]
 
+    def __post_init__(self):
+        """
+        Test the distribution for consistency.
+        """
+        for path in self.distribution:
+            assert len(path) <= self.limit
+        seen = set()
+        fringe = [((0, 0),)]
+        while fringe:
+            path = fringe.pop()
+            assert path in self.distribution, f"Path {path} not in distribution"
+            seen.add(path)
+            for parent_index, _ in self.distribution[path]:
+                for position in range(self.symbols[parent_index][1]):
+                    new_path = path + ((parent_index, position),)
+                    new_path = new_path[-self.limit :]
+                    if new_path not in seen:
+                        fringe.append(new_path)
+
     @cached_property
     def symbol_to_index(self) -> Dict[str, int]:
+        """
+        Dictionary mapping symbols to their indices.
+        """
         return {symbol: i for i, (symbol, _) in enumerate(self.symbols)}
 
     @cached_property
     def index_within_distribution_list(
         self,
     ) -> Dict[Tuple[Tuple[int, int], ...], Dict[int, int]]:
+        """
+        Index within the distribution list for each production, for each path.
+        (The dependence on the path is because the distribution lists do not
+        include productions with -inf likelihood.)
+        """
         return {
             k: {x: i for i, (x, _) in enumerate(v)}
             for k, v in self.distribution.items()
@@ -56,12 +92,18 @@ class TreeDistribution:
 
     @cached_property
     def distribution_dict(self) -> Dict[Tuple[Tuple[int, int], ...], Dict[int, float]]:
+        """
+        Distributions, as dictionaries.
+        """
         return {k: dict(v) for k, v in self.distribution.items()}
 
     @cached_property
     def likelihood_arrays(
         self,
     ) -> Dict[Tuple[Tuple[int, int], ...], Tuple[np.ndarray, np.ndarray]]:
+        """
+        Distributions, as arrays.
+        """
         return {
             k: (
                 np.array([x[0] for x in v]),
@@ -74,6 +116,9 @@ class TreeDistribution:
     def sampling_dict_arrays(
         self,
     ) -> Dict[Tuple[Tuple[int, int], ...], Tuple[np.ndarray, np.ndarray]]:
+        """
+        Like likelihood_arrays, but with probabilities instead of log probabilities.
+        """
         return {
             k: (syms, np.exp(log_probs))
             for k, (syms, log_probs) in self.likelihood_arrays.items()
@@ -81,12 +126,16 @@ class TreeDistribution:
 
     @cached_property
     def ordering(self) -> NodeOrdering:
+        """
+        Get the node ordering for this tree distribution.
+        """
         return self.node_ordering(self)
 
 
 class TreeProgramDistributionFamily(ProgramDistributionFamily):
     """
-    See `tree_dist_enumerator.py` for more information.
+    Represents a family of program distributions that can be represented as trees.
+    Uses the various methods of ``TreeDistribution`` to compute likelihoods, samples, etc.
     """
 
     @abstractmethod
@@ -96,15 +145,15 @@ class TreeProgramDistributionFamily(ProgramDistributionFamily):
         """
         Returns a tree distribution representing the given program distribution.
 
-        If `distribution` is `None`, returns a tree distribution with all fields
-            initialized, but the probabilities are not guaranteed to have any properties.
-            This is useful for tasks where you want the skeleton of the tree distribution,
-            but don't need the actual distribution.
+        If ``distribution`` is ``None``, returns a tree distribution with all fields
+        initialized, but the probabilities are not guaranteed to have any properties.
+        This is useful for tasks where you want the skeleton of the tree distribution,
+        but don't need the actual distribution.
         """
 
     def tree_distribution(self, distribution: ProgramDistribution) -> TreeDistribution:
         """
-        Cached version of `compute_tree_distribution`.
+        Cached version of ``compute_tree_distribution``.
         """
         # This is a bit of a hack, but it reduces the need to pass around
         # the tree distribution everywhere, or to compute it multiple times.
@@ -118,7 +167,7 @@ class TreeProgramDistributionFamily(ProgramDistributionFamily):
     @cached_property
     def tree_distribution_skeleton(self) -> TreeDistribution:
         """
-        Cached version of `compute_tree_distribution(None)`.
+        Cached version of ``compute_tree_distribution(None)``.
         """
 
         return self.compute_tree_distribution(None)
@@ -147,9 +196,6 @@ class TreeProgramDistributionFamily(ProgramDistributionFamily):
         program: SExpression,
         tracker: Union[NoneType, Callable[[SExpression, float], NoneType]] = None,
     ) -> float:
-        """
-        Compute the likelihood of a program under a distribution.
-        """
         # pylint: disable=cyclic-import
         from .tree_dist_likelihood_computer import compute_likelihood
 
@@ -164,7 +210,11 @@ class TreeProgramDistributionFamily(ProgramDistributionFamily):
         program: SExpression,
     ) -> Dict[SExpression, float]:
         """
-        Compute the likelihood of a program under a distribution.
+        Compute the likelihood of each node in the program, given the program distribution.
+
+        :param dist: The program distribution.
+        :param program: The program to compute the likelihoods for.
+        :return: A dictionary mapping nodes to their contributions to the likelihood.
         """
         likelihoods = []
 
