@@ -1,26 +1,24 @@
 import queue
 from dataclasses import dataclass, field
-from typing import Callable
+from typing import Iterable, TypeVar
 
-from neurosym.programs.s_expression import SExpression
 from neurosym.search_graph.search_graph import SearchGraph
 
+X = TypeVar("X")
 
-def astar(g: SearchGraph, cost_plus_heuristic: Callable[[SExpression], float]):
+
+def astar(g: SearchGraph[X]) -> Iterable[X]:
     """
-    Performs a breadth-first search on the given search graph, yielding each node in the
-    order it was visited.
+    Performs an A* search on the given search graph, yielding each goal node in the
+    order it was visited. Requires that the search graph implement a cost method.
 
     :param g: Search graph to search over
-    :param cost_plus_heuristic: Cost plus heuristic function to use for A*.
-        The heuristic function should be admissible, i.e. it should never overestimate
-        the cost to reach the goal.
     """
     visited = set()
     fringe = queue.PriorityQueue()
 
     def add_to_fringe(node):
-        fringe.put(AStarNode(cost_plus_heuristic(node), node))
+        fringe.put(_AStarNode(g.cost(node), node))
 
     add_to_fringe(g.initial_node())
     # this is similar to the BFS algorithm
@@ -30,17 +28,16 @@ def astar(g: SearchGraph, cost_plus_heuristic: Callable[[SExpression], float]):
         if node in visited:
             continue
         visited.add(node)
-        if g.is_goal_node(node):
-            yield node
+        yield from g.yield_goal_node(node)
         for child in g.expand_node(node):
             add_to_fringe(child)
 
 
 @dataclass(order=True)
-class AStarNode:
+class _AStarNode:
     """
     Represents a node in the A* search tree.
     """
 
     cost: float
-    node: SExpression = field(compare=False)
+    node: X = field(compare=False)
