@@ -1,12 +1,17 @@
+from typing import Iterable, TypeVar
+
 from neurosym.search_graph.search_graph import SearchGraph
 
+X = TypeVar("X")
 
-def bfs(g: SearchGraph, iteration_limit=float("inf")):
+
+def bfs(g: SearchGraph[X], iteration_limit=float("inf")) -> Iterable[X]:
     """
-    Performs a breadth-first search on the given search graph, yielding each node in the
-    order it was visited.
+    Performs a breadth-first search on the given search graph, yielding each goal node
+    in the order it was visited.
 
     :param g: Search graph to search over
+    :param iteration_limit: Maximum number of iterations to perform
     """
     visited = set()
     queue = [g.initial_node()]
@@ -18,7 +23,6 @@ def bfs(g: SearchGraph, iteration_limit=float("inf")):
         if node in visited:
             continue
         visited.add(node)
-        if g.is_goal_node(node):
-            yield node
+        yield from g.yield_goal_node(node)
         for child in g.expand_node(node):
             queue.append(child)
